@@ -19,7 +19,7 @@
 
 
 #include "exiv2extractor.h"
-#include <QDebug>
+#include <cmath>
 
 using namespace KFileMetaData;
 
@@ -32,11 +32,16 @@ Exiv2Extractor::Exiv2Extractor(QObject* parent)
 namespace
 {
 static const QStringList supportedMimeTypes = {
+    QStringLiteral("image/bmp"),
+    QStringLiteral("image/gif"),
     QStringLiteral("image/jp2"),
     QStringLiteral("image/jpeg"),
     QStringLiteral("image/pgf"),
     QStringLiteral("image/png"),
     QStringLiteral("image/tiff"),
+#ifdef HAVE_WEBP_SUPPORT
+    QStringLiteral("image/webp"),
+#endif
     QStringLiteral("image/x-exv"),
     QStringLiteral("image/x-canon-cr2"),
     QStringLiteral("image/x-canon-crw"),
@@ -48,6 +53,7 @@ static const QStringList supportedMimeTypes = {
     QStringLiteral("image/x-pentax-pef"),
     QStringLiteral("image/x-photoshop"),
     QStringLiteral("image/x-samsung-srw"),
+    QStringLiteral("image/x-tga"),
 };
 
 QString toString(const Exiv2::Value& value)
@@ -142,7 +148,11 @@ void Exiv2Extractor::extract(ExtractionResult* result)
     QByteArray arr = result->inputUrl().toUtf8();
     std::string fileString(arr.data(), arr.length());
 
+#if EXIV2_TEST_VERSION(0, 28, 0)
+    Exiv2::Image::UniquePtr image;
+#else
     Exiv2::Image::AutoPtr image;
+#endif
     try {
         image = Exiv2::ImageFactory::open(fileString);
     } catch (const std::exception&) {
@@ -174,13 +184,17 @@ void Exiv2Extractor::extract(ExtractionResult* result)
 
     const Exiv2::ExifData& data = image->exifData();
 
-    add(result, data, Property::ImageMake, "Exif.Image.Make", QVariant::String);
-    add(result, data, Property::ImageModel, "Exif.Image.Model", QVariant::String);
+    add(result, data, Property::Manufacturer, "Exif.Image.Make", QVariant::String);
+    add(result, data, Property::Model, "Exif.Image.Model", QVariant::String);
+    add(result, data, Property::Description, "Exif.Image.ImageDescription", QVariant::String);
+    add(result, data, Property::Artist, "Exif.Image.Artist", QVariant::String);
+    add(result, data, Property::Copyright, "Exif.Image.Copyright", QVariant::String);
+    add(result, data, Property::Generator, "Exif.Image.Software", QVariant::String);
     add(result, data, Property::ImageDateTime, "Exif.Image.DateTime", QVariant::DateTime);
     add(result, data, Property::ImageOrientation, "Exif.Image.Orientation", QVariant::Int);
     add(result, data, Property::PhotoFlash, "Exif.Photo.Flash", QVariant::Int);
     add(result, data, Property::PhotoPixelXDimension, "Exif.Photo.PixelXDimension", QVariant::Int);
-    add(result, data, Property::PhotoPixelXDimension, "Exif.Photo.PixelYDimension", QVariant::Int);
+    add(result, data, Property::PhotoPixelYDimension, "Exif.Photo.PixelYDimension", QVariant::Int);
     add(result, data, Property::PhotoDateTimeOriginal, "Exif.Photo.DateTimeOriginal", QVariant::DateTime);
     add(result, data, Property::PhotoFocalLength, "Exif.Photo.FocalLength", QVariant::Double);
     add(result, data, Property::PhotoFocalLengthIn35mmFilm, "Exif.Photo.FocalLengthIn35mmFilm", QVariant::Double);
@@ -196,7 +210,7 @@ void Exiv2Extractor::extract(ExtractionResult* result)
 
     double latitude = fetchGpsDouble(data, "Exif.GPSInfo.GPSLatitude");
     double longitude = fetchGpsDouble(data, "Exif.GPSInfo.GPSLongitude");
-    double altitude = fetchGpsDouble(data, "Exif.GPSInfo.GPSAltitude");
+    double altitude = fetchGpsAltitude(data);
 
     QByteArray latRef = fetchByteArray(data, "Exif.GPSInfo.GPSLatitudeRef");
     if (!latRef.isEmpty() && latRef[0] == 'S')
@@ -206,15 +220,15 @@ void Exiv2Extractor::extract(ExtractionResult* result)
     if (!longRef.isEmpty() && longRef[0] == 'W')
         longitude *= -1;
 
-    if (latitude) {
+    if (!std::isnan(latitude)) {
         result->add(Property::PhotoGpsLatitude, latitude);
     }
 
-    if (longitude) {
+    if (!std::isnan(longitude)) {
         result->add(Property::PhotoGpsLongitude, longitude);
     }
 
-    if (altitude) {
+    if (!std::isnan(altitude)) {
         result->add(Property::PhotoGpsAltitude, altitude);
     }
 }
@@ -241,8 +255,8 @@ double Exiv2Extractor::fetchGpsDouble(const Exiv2::ExifData& data, const char* n
         n = (*it).toRational(0).first;
         d = (*it).toRational(0).second;
 
-        if (d == 0) {
-            return 0.0;
+        if (d == 0.0) {
+            return std::numeric_limits<double>::quiet_NaN();
         }
 
         double deg = n / d;
@@ -250,7 +264,7 @@ double Exiv2Extractor::fetchGpsDouble(const Exiv2::ExifData& data, const char* n
         n = (*it).toRational(1).first;
         d = (*it).toRational(1).second;
 
-        if (d == 0) {
+        if (d == 0.0) {
             return deg;
         }
 
@@ -262,7 +276,7 @@ double Exiv2Extractor::fetchGpsDouble(const Exiv2::ExifData& data, const char* n
         n = (*it).toRational(2).first;
         d = (*it).toRational(2).second;
 
-        if (d == 0) {
+        if (d == 0.0) {
             return deg;
         }
 
@@ -274,13 +288,37 @@ double Exiv2Extractor::fetchGpsDouble(const Exiv2::ExifData& data, const char* n
         return deg;
     }
 
-    return 0.0;
+    return std::numeric_limits<double>::quiet_NaN();
+}
+
+double Exiv2Extractor::fetchGpsAltitude(const Exiv2::ExifData& data)
+{
+    double alt = std::numeric_limits<double>::quiet_NaN();
+    Exiv2::ExifData::const_iterator it = data.findKey(Exiv2::ExifKey("Exif.GPSInfo.GPSAltitude"));
+    if (it != data.end() && it->count() > 0 &&
+        (it->value().typeId() == Exiv2::unsignedRational || it->value().typeId() == Exiv2::signedRational)) {
+        auto ratio = it->value().toRational();
+        if (ratio.second == 0) {
+            return alt;
+        }
+        it = data.findKey(Exiv2::ExifKey("Exif.GPSInfo.GPSAltitudeRef"));
+        if (it != data.end() && it->count() > 0 &&
+            (it->value().typeId() == Exiv2::unsignedByte || it->value().typeId() == Exiv2::signedByte)) {
+            auto altRef = it->value().toLong();
+            if (altRef) {
+                alt = -1.0 * ratio.first / ratio.second;
+            } else {
+                alt = 1.0 * ratio.first / ratio.second;
+            }
+        }
+    }
+    return alt;
 }
 
 QByteArray Exiv2Extractor::fetchByteArray(const Exiv2::ExifData& data, const char* name)
 {
     Exiv2::ExifData::const_iterator it = data.findKey(Exiv2::ExifKey(name));
-    if (it != data.end()) {
+    if (it != data.end() && it->count() > 0) {
         std::string str = it->value().toString();
         return QByteArray(str.c_str(), str.size());
     }
